Package: CoreGx
Type: Package
Title: Classes and Functions to Serve as the Basis for Other 'Gx'
        Packages
Version: 2.14.0
Date: 2024-03-11
Authors@R: c(
    person(given="Jermiah", family="Joseph", email="jermiah.joseph@uhn.ca",
        role=c("aut")),
    person(given="Petr", family="Smirnov", email = "petr.smirnov@uhnresearch.ca",
        role = c("aut")),
    person(given="Ian", family="Smith", email = "ianc.smith@mail.utoronto.ca",
        role = c("aut")),
    person(given="Christopher", family="Eeles", email = "christopher.eeles@uhnresearch.ca",
        role = c("aut")),
    person(given="Feifei", family="Li", email="ff.li@mail.utoronto.ca",
        role=c("aut")),
    person("Benjamin", "Haibe-Kains",
        email = "benjamin.haibe.kains@utoronto.ca", role = c("aut", "cre"))
    )
Description: A collection of functions and classes which serve as the foundation
    for our lab's suite of R packages, such as 'PharmacoGx' and 'RadioGx'. This
    package was created to abstract shared functionality from other lab package
    releases to increase ease of maintainability and reduce code repetition in
    current and future 'Gx' suite programs. Major features include a 'CoreSet'
    class, from which 'RadioSet' and 'PharmacoSet' are derived, along with get
    and set methods for each respective slot. Additional functions related to
    fitting and plotting dose response curves, quantifying statistical
    correlation and calculating area under the curve (AUC) or survival fraction
    (SF) are included. For more details please see the included documentation,
    as well as:
    Smirnov, P., Safikhani, Z., El-Hachem, N., Wang, D., She, A., Olsen, C.,
        Freeman, M., Selby, H., Gendoo, D., Grossman, P., Beck, A., Aerts, H.,
        Lupien, M., Goldenberg, A. (2015) <doi:10.1093/bioinformatics/btv723>.
    Manem, V., Labie, M., Smirnov, P., Kofia, V., Freeman, M., Koritzinksy, M.,
        Abazeed, M., Haibe-Kains, B., Bratman, S. (2018) <doi:10.1101/449793>.
VignetteBuilder: knitr
VignetteEngine: knitr::rmarkdown
biocViews: Software, Pharmacogenomics, Classification, Survival
Encoding: UTF-8
LazyData: TRUE
Depends: R (>= 4.1), BiocGenerics, SummarizedExperiment
Imports: Biobase, S4Vectors, MultiAssayExperiment, MatrixGenerics,
        piano, BiocParallel, parallel, BumpyMatrix, checkmate, methods,
        stats, utils, graphics, grDevices, lsa, data.table, crayon,
        glue, rlang, bench
Suggests: pander, markdown, BiocStyle, rmarkdown, knitr, formatR,
        testthat
License: GPL (>= 3)
Config/testthat/edition: 3
Roxygen: list(markdown=TRUE, r6=FALSE)
RoxygenNote: 7.3.1
RCollate: 'allGenerics.R' 'immutable-class.R' 'LongTable-class.R'
        'CoreSet-class.R' 'CoreSet-accessors.R' 'CoreSet-utils.R'
        'DataMapper-class.R' 'LongTable-accessors.R'
        'LongTable-utils.R' 'LongTableDataMapper-class.R'
        'LongTableDataMapper-accessors.R'
        'TreatmentResponseExperiment-class.R' 'TREDataMapper-class.R'
        'adaptiveMatthewCor.R' 'aggregate-methods.R'
        'callingWaterfall.R' 'connectivityScore.R' 'cosinePerm.R'
        'datasets.R' 'deprecated.R' 'endoaggregate-methods.R'
        'globals.R' 'gwc.R' 'matthewCor.R' 'mergeAssays-method.R'
        'methods-coerce.R' 'methods-dim.R' 'methods-dimnames.R'
        'methods-drugSensitivitySig.R' 'methods-guessMapping.R'
        'methods-metaConstruct.R' 'methods-subsetTo.R'
        'optimizeCoreGx.R' 'updateObject-methods.R' 'utilities.R'
        'utils-iteration.R' 'utils-messages.R' 'utils-optimization.R'
        'utils-testing.R' 'utils-updateS4.R'
Collate: 'allGenerics.R' 'immutable-class.R' 'LongTable-class.R'
        'CoreSet-class.R' 'CoreSet-accessors.R' 'CoreSet-utils.R'
        'DataMapper-class.R' 'LongTable-accessors.R'
        'LongTable-utils.R' 'LongTableDataMapper-class.R'
        'LongTableDataMapper-accessors.R'
        'TreatmentResponseExperiment-class.R' 'TREDataMapper-class.R'
        'adaptiveMatthewCor.R' 'aggregate-methods.R'
        'callingWaterfall.R' 'connectivityScore.R' 'cosinePerm.R'
        'datasets.R' 'deprecated.R' 'endoaggregate-methods.R'
        'globals.R' 'gwc.R' 'matthewCor.R' 'mergeAssays-method.R'
        'methods-coerce.R' 'methods-dim.R' 'methods-dimnames.R'
        'methods-drugSensitivitySig.R' 'methods-guessMapping.R'
        'methods-metaConstruct.R' 'methods-subsetTo.R'
        'optimizeCoreGx.R' 'updateObject-methods.R' 'utilities.R'
        'utils-iteration.R' 'utils-messages.R' 'utils-optimization.R'
        'utils-testing.R' 'utils-updateS4.R'
git_url: https://git.bioconductor.org/packages/CoreGx
git_branch: RELEASE_3_22
git_last_commit: d18e3bb
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 03:18:39 UTC; biocbuild
Author: Jermiah Joseph [aut],
  Petr Smirnov [aut],
  Ian Smith [aut],
  Christopher Eeles [aut],
  Feifei Li [aut],
  Benjamin Haibe-Kains [aut, cre]
Maintainer: Benjamin Haibe-Kains <benjamin.haibe.kains@utoronto.ca>
