# Generated by roxygen2: do not edit by hand

S3method("$",immutable)
S3method("$<-",immutable)
S3method("[",immutable)
S3method("[<-",immutable)
S3method("[[",immutable)
S3method("[[<-",immutable)
S3method("dimnames<-",immutable)
S3method("names<-",immutable)
S3method(Ops,immutable)
S3method(c,immutable)
S3method(mutable,default)
S3method(print,immutable)
export("assayMap<-")
export("cellInfo<-")
export("cellNames<-")
export("colDataMap<-")
export("colnames<-.immutable")
export("curation<-")
export("datasetType<-")
export("dateCreated<-")
export("fNames<-")
export("featureInfo<-")
export("mDataNames<-")
export("metadataMap<-")
export("molecularProfiles<-")
export("molecularProfilesSlot<-")
export("name<-")
export("pertNumber<-")
export("phenoInfo<-")
export("rawdata<-")
export("rowDataMap<-")
export("rownames<-.immutable")
export("sampleInfo<-")
export("sensNumber<-")
export("sensitivityInfo<-")
export("sensitivityMeasures<-")
export("sensitivityProfiles<-")
export("sensitivityRaw<-")
export("sensitivitySlot<-")
export("subset<-.immutable")
export("treatmentInfo<-")
export("treatmentNames<-")
export("treatmentResponse<-")
export(.)
export(.S4MethodContext)
export(.all_equals)
export(.collapse)
export(.distancePointLine)
export(.distancePointSegment)
export(.dmedncauchys)
export(.dmednnormals)
export(.edmedncauchys)
export(.edmednnormals)
export(.error)
export(.errorMsg)
export(.examineGOF)
export(.fitCurve)
export(.fitCurve2)
export(.formatMessage)
export(.funContext)
export(.getSupportVec)
export(.intersectList)
export(.length_unique)
export(.linInd)
export(.list_unique)
export(.magnitude)
export(.matInd)
export(.meshEval)
export(.meshEval2)
export(.message)
export(.multinom)
export(.parseToRoxygen)
export(.patternSearch)
export(.patternSearch2)
export(.pmedncauchys)
export(.pmednnormals)
export(.reformatData)
export(.residual)
export(.rmedncauchys)
export(.rmednnormals)
export(.sanitizeInput)
export(.stripWhiteSpace)
export(.summarizeMolecularNumbers)
export(.symSetDiffList)
export(.unionList)
export(.warnMsg)
export(.warning)
export(CoreSet)
export(CoreSet2)
export(LongTable)
export(LongTableDataMapper)
export(TREDataMapper)
export(TreatmentResponseExperiment)
export(aggregate2)
export(amcc)
export(as)
export(as.long.table)
export(assayCols)
export(assayIndex)
export(assayKeys)
export(assayMap)
export(buildLongTable)
export(callingWaterfall)
export(cardinality)
export(cellInfo)
export(cellNames)
export(checkColumnCardinality)
export(checkCsetStructure)
export(colDataMap)
export(colIDs)
export(colMeta)
export(collect_fn_params)
export(connectivityScore)
export(cosinePerm)
export(curation)
export(datasetType)
export(dateCreated)
export(drop_fn_params)
export(drugSensitivitySig)
export(endoaggregate)
export(fNames)
export(featureInfo)
export(getIntern)
export(guessMapping)
export(gwc)
export(idCols)
export(immutable)
export(is.immutable)
export(is.items)
export(is_optim_compatible)
export(mDataNames)
export(make_optim_function)
export(mcc)
export(metaConstruct)
export(metadataMap)
export(molecularProfiles)
export(molecularProfilesSlot)
export(mutable)
export(name)
export(optimizeCoreGx)
export(pertNumber)
export(phenoInfo)
export(rawdata)
export(reindex)
export(rowDataMap)
export(rowIDs)
export(rowMeta)
export(sampleInfo)
export(sensNumber)
export(sensitivityInfo)
export(sensitivityMeasures)
export(sensitivityProfiles)
export(sensitivityRaw)
export(sensitivitySlot)
export(sensitivitySlotToLongTable)
export(show.immutable)
export(showSigAnnot)
export(subset.immutable)
export(subsetByFeature)
export(subsetBySample)
export(subsetByTreatment)
export(subsetTo)
export(summarizeMolecularProfiles)
export(summarizeSensitivityProfiles)
export(treatmentInfo)
export(treatmentNames)
export(treatmentResponse)
export(updateSampleId)
export(updateTreatmentId)
exportClasses(CoreSet)
exportClasses(LongTable)
exportClasses(TreatmentResponseExperiment)
exportClasses(immutable_list)
exportMethods("$")
exportMethods("$<-")
exportMethods("[")
exportMethods("[[")
exportMethods("[[<-")
exportMethods("annotation<-")
exportMethods("assay<-")
exportMethods("assayMap<-")
exportMethods("assays<-")
exportMethods("colData<-")
exportMethods("colDataMap<-")
exportMethods("curation<-")
exportMethods("datasetType<-")
exportMethods("dateCreated<-")
exportMethods("fNames<-")
exportMethods("featureInfo<-")
exportMethods("mDataNames<-")
exportMethods("metadata<-")
exportMethods("metadataMap<-")
exportMethods("molecularProfiles<-")
exportMethods("molecularProfilesSlot<-")
exportMethods("name<-")
exportMethods("pertNumber<-")
exportMethods("phenoInfo<-")
exportMethods("rawdata<-")
exportMethods("rowData<-")
exportMethods("rowDataMap<-")
exportMethods("sampleInfo<-")
exportMethods("sampleNames<-")
exportMethods("sensNumber<-")
exportMethods("sensitivityInfo<-")
exportMethods("sensitivityProfiles<-")
exportMethods("sensitivityRaw<-")
exportMethods("treatmentInfo<-")
exportMethods("treatmentNames<-")
exportMethods("treatmentResponse<-")
exportMethods(aggregate)
exportMethods(annotation)
exportMethods(assay)
exportMethods(assayCols)
exportMethods(assayIndex)
exportMethods(assayKeys)
exportMethods(assayMap)
exportMethods(assayNames)
exportMethods(assays)
exportMethods(buildComboProfiles)
exportMethods(buildLongTable)
exportMethods(colData)
exportMethods(colDataMap)
exportMethods(colIDs)
exportMethods(colMeta)
exportMethods(colnames)
exportMethods(curation)
exportMethods(datasetType)
exportMethods(dateCreated)
exportMethods(dim)
exportMethods(dimnames)
exportMethods(endoaggregate)
exportMethods(fNames)
exportMethods(featureInfo)
exportMethods(getIntern)
exportMethods(guessMapping)
exportMethods(idCols)
exportMethods(lapply)
exportMethods(mDataNames)
exportMethods(mergeAssays)
exportMethods(metaConstruct)
exportMethods(metadata)
exportMethods(metadataMap)
exportMethods(molecularProfiles)
exportMethods(molecularProfilesSlot)
exportMethods(name)
exportMethods(names)
exportMethods(pertNumber)
exportMethods(phenoInfo)
exportMethods(rawdata)
exportMethods(reindex)
exportMethods(rowData)
exportMethods(rowDataMap)
exportMethods(rowIDs)
exportMethods(rowMeta)
exportMethods(rownames)
exportMethods(sampleInfo)
exportMethods(sampleNames)
exportMethods(sensNumber)
exportMethods(sensitivityInfo)
exportMethods(sensitivityMeasures)
exportMethods(sensitivityProfiles)
exportMethods(sensitivityRaw)
exportMethods(show)
exportMethods(subset)
exportMethods(subsetByFeature)
exportMethods(subsetBySample)
exportMethods(subsetByTreatment)
exportMethods(treatmentInfo)
exportMethods(treatmentNames)
exportMethods(treatmentResponse)
exportMethods(updateObject)
import(checkmate)
import(data.table)
import(methods)
importClassesFrom(MultiAssayExperiment,MultiAssayExperiment)
importClassesFrom(S4Vectors,Annotated)
importClassesFrom(S4Vectors,DataFrame)
importClassesFrom(S4Vectors,List)
importClassesFrom(S4Vectors,list_OR_List)
importFrom(Biobase,annotation)
importFrom(Biobase,assayDataElementNames)
importFrom(Biobase,exprs)
importFrom(Biobase,fData)
importFrom(Biobase,pData)
importFrom(Biobase,protocolData)
importFrom(BiocGenerics,"%in%")
importFrom(BiocGenerics,match)
importFrom(BiocParallel,"bpprogressbar<-")
importFrom(BiocParallel,"bpworkers<-")
importFrom(BiocParallel,bplapply)
importFrom(BiocParallel,bpparam)
importFrom(BiocParallel,bpprogressbar)
importFrom(BiocParallel,bpworkers)
importFrom(BumpyMatrix,splitAsBumpyMatrix)
importFrom(MatrixGenerics,colAlls)
importFrom(MatrixGenerics,rowAnys)
importFrom(MultiAssayExperiment,"experiments<-")
importFrom(MultiAssayExperiment,MultiAssayExperiment)
importFrom(MultiAssayExperiment,experiments)
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,List)
importFrom(S4Vectors,SimpleList)
importFrom(S4Vectors,endoapply)
importFrom(S4Vectors,metadata)
importFrom(SummarizedExperiment,"assayNames<-")
importFrom(SummarizedExperiment,"colData<-")
importFrom(SummarizedExperiment,"rowData<-")
importFrom(SummarizedExperiment,Assays)
importFrom(SummarizedExperiment,SummarizedExperiment)
importFrom(SummarizedExperiment,`rowData<-`)
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,assayNames)
importFrom(SummarizedExperiment,assays)
importFrom(SummarizedExperiment,colData)
importFrom(SummarizedExperiment,rowData)
importFrom(bench,mark)
importFrom(bench,press)
importFrom(checkmate,assert)
importFrom(checkmate,assertCharacter)
importFrom(checkmate,assertClass)
importFrom(checkmate,assertDataFrame)
importFrom(checkmate,assertDataTable)
importFrom(checkmate,assertList)
importFrom(checkmate,assertLogical)
importFrom(checkmate,assertSubset)
importFrom(compiler,cmpfun)
importFrom(crayon,"%+%")
importFrom(crayon,blue)
importFrom(crayon,cyan)
importFrom(crayon,green)
importFrom(crayon,magenta)
importFrom(crayon,red)
importFrom(crayon,yellow)
importFrom(data.table,":=")
importFrom(data.table,`:=`)
importFrom(data.table,as.data.table)
importFrom(data.table,copy)
importFrom(data.table,data.table)
importFrom(data.table,dcast)
importFrom(data.table,fsetdiff)
importFrom(data.table,key)
importFrom(data.table,melt.data.table)
importFrom(data.table,merge.data.table)
importFrom(data.table,rbindlist)
importFrom(data.table,set)
importFrom(data.table,setDF)
importFrom(data.table,setDT)
importFrom(data.table,setDTthreads)
importFrom(data.table,setcolorder)
importFrom(data.table,setindexv)
importFrom(data.table,setkeyv)
importFrom(data.table,setnames)
importFrom(data.table,tstrsplit)
importFrom(grDevices,dev.off)
importFrom(grDevices,pdf)
importFrom(grDevices,rainbow)
importFrom(graphics,abline)
importFrom(graphics,hist)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(graphics,points)
importFrom(lsa,cosine)
importFrom(methods,getPackageName)
importFrom(methods,is)
importFrom(parallel,detectCores)
importFrom(piano,loadGSC)
importFrom(piano,runGSA)
importFrom(rlang,trace_back)
importFrom(stats,complete.cases)
importFrom(stats,cor.test)
importFrom(stats,cov.wt)
importFrom(stats,dcauchy)
importFrom(stats,dnorm)
importFrom(stats,integrate)
importFrom(stats,lm)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,optim)
importFrom(stats,pcauchy)
importFrom(stats,pchisq)
importFrom(stats,pnorm)
importFrom(stats,pt)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,rcauchy)
importFrom(stats,rnorm)
importFrom(stats,setNames)
importFrom(stats,var)
importFrom(utils,packageName)
importFrom(utils,sessionInfo)
importMethodsFrom(Biobase,"sampleNames<-")
importMethodsFrom(Biobase,dimnames)
importMethodsFrom(Biobase,sampleNames)
importMethodsFrom(BiocGenerics,"annotation<-")
importMethodsFrom(BiocGenerics,annotation)
importMethodsFrom(BiocGenerics,lapply)
importMethodsFrom(BiocGenerics,subset)
importMethodsFrom(BiocGenerics,updateObject)
importMethodsFrom(S4Vectors,"metadata<-")
importMethodsFrom(S4Vectors,aggregate)
importMethodsFrom(S4Vectors,metadata)
importMethodsFrom(SummarizedExperiment,"assay<-")
importMethodsFrom(SummarizedExperiment,"assays<-")
importMethodsFrom(SummarizedExperiment,assay)
importMethodsFrom(SummarizedExperiment,assayNames)
importMethodsFrom(SummarizedExperiment,assays)
importMethodsFrom(SummarizedExperiment,dim)
