% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/allGenerics.R
\name{rowIDs}
\alias{rowIDs}
\title{Generic to access the row identifiers from}
\usage{
rowIDs(object, ...)
}
\arguments{
\item{object}{\code{S4} An object to get row id columns from.}

\item{...}{Allow new arguments to this generic.}
}
\value{
Depends on the implemented method.
}
\description{
Generic to access the row identifiers from
}
\examples{
print("Generics shouldn't need examples?")

}
