% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LongTable-utils.R
\name{.}
\alias{.}
\title{Convenience function for converting R code to a call}
\usage{
.(...)
}
\arguments{
\item{...}{\code{pairlist} One or more R expressions to convert to calls.}
}
\value{
\code{call} An R call object containing the quoted expression.
}
\description{
This is used to pass through unevaluated R expressions into subset and
\code{[}, where they will be evaluated in the correct context.
}
\examples{
.(sample_line1 == 'A2058')

}
